use std::env;
use std::fs::File;
use std::io::{prelude::*, BufWriter};
use std::path::Path;

use gl_generator::{Api, Fallbacks, Profile, Registry, StructGenerator};

// Pinned OpenGL version used by Miru
const MIRU_PINNED_VERSION: (u8, u8) = (3, 3);

fn main() -> std::io::Result<()> {
    let out_dir = env::var("OUT_DIR").unwrap();

    // Generate pinned version file constant(s) for user code
    let opengl_version_filename = Path::new(&out_dir).join("miru-pinned-opengl-version.rs");
    let file = File::create(opengl_version_filename)?;
    let mut file = BufWriter::new(file);

    writeln!(
        &mut file,
        "pub const OPENGL_VERSION: (u8, u8) = {:?};",
        MIRU_PINNED_VERSION
    )?;

    // Generate OpenGL bindings for pinned version
    let opengl_binding_filename = Path::new(&out_dir).join("opengl-bindings.rs");
    let mut file = File::create(opengl_binding_filename)?;

    Registry::new(
        Api::Gl,
        MIRU_PINNED_VERSION,
        Profile::Core,
        Fallbacks::All,
        [],
    )
    .write_bindings(StructGenerator, &mut file)?;

    Ok(())
}
